/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2011-2016, High Performance Humanoid Technologies (H2T), Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package
 * @author
 * @date
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#pragma once

#include <ArmarXCore/interface/core/PackagePath.ice>

#include <RobotAPI/interface/armem/mns.ice>
#include <RobotAPI/interface/skills/SkillManagerInterface.ice>
#include <RobotAPI/interface/units/HandUnitInterface.ice>
#include <RobotAPI/interface/units/KinematicUnitInterface.ice>
#include <RobotAPI/interface/units/LocalizationUnitInterface.ice>
#include <RobotAPI/interface/units/PlatformUnitInterface.ice>

module armarx
{
    module robot_name_service
    {
        module dto
        {

            struct Hand
            {
                string name;
                string ft_name;
                HandUnitInterface* handUnitInterface;
            };

            dictionary<string, Hand> Hands;

            struct Arm
            {
                string kinematicChainName;
                Hand hand;
            };

            dictionary<string, Arm> Arms;

            struct Robot
            {
                string name;
                armarx::data::PackagePath xmlPackagePath;

                // Memory and skills. MAY BE NULL
                armarx::armem::mns::MemoryNameSystemInterface* memoryNameSystem;
                armarx::skills::manager::dti::SkillManagerInterface* skillManager;

                // kinematic stuff. MAY BE EMPTY
                Arms arms;

                // units. MAY BE NULL
                armarx::KinematicUnitInterface* kinematicUnitInterface;
                armarx::PlatformUnitInterface* platformUnitInterface;
                armarx::LocalizationUnitInterface* localizationUnitInterface;

                // TODO: Feel free to extend!
            };
        };

        module dti
        {
            interface RobotNameServiceInterface
            {
                bool registerRobot(dto::Robot robot);
                void unregisterRobot(string name);
                optional(1) dto::Robot getRobot(string name);
            };
        };
    };
};
