#pragma once

#include <memory>
#include <string>

#include <SimoxUtility/shapes/OrientedBox.h>

#include <RobotAPI/libraries/core/FramedPose.h>

namespace armarx::priorknowledge::util
{
    enum class LocationType
    {
        FRAMED_LOCATION,
        FRAMED_BOXED_LOCATION
    };

    struct LocationId
    {
        std::string source;
        std::string name;

        LocationId(const std::string& s, const std::string& n) : source(s), name(n)
        {
        }

        std::string toString() const;
    };

    struct Location
    {
        LocationId id;
        LocationType type;

        Location(const LocationId& i, const LocationType t) : id(i), type(t)
        {
        }

        virtual ~Location() = default;
    };

    struct FramedLocation : public Location
    {
        std::string frame;
        std::string agent;
        Eigen::Matrix4f pose;

        FramedLocation(const LocationId& i,
                       const LocationType t,
                       const std::string& f,
                       const std::string& a,
                       const Eigen::Matrix4f& p) :
            Location(i, t), frame(f), agent(a), pose(p)
        {
        }

        virtual ~FramedLocation() = default;

        armarx::FramedPose toFramedPose();
    };

    struct FramedBoxedLocation : public FramedLocation
    {
        Eigen::Vector3f extents;

        FramedBoxedLocation(const LocationId& i,
                            const LocationType t,
                            const std::string& f,
                            const std::string& a,
                            const Eigen::Matrix4f& p,
                            const Eigen::Vector3f& e) :
            FramedLocation(i, t, f, a, p), extents(e)
        {
        }

        virtual ~FramedBoxedLocation() = default;

        simox::OrientedBox<float> toOrientedBox();
    };

    using FramedBoxedLocationPtr = std::unique_ptr<FramedBoxedLocation>;
    using FramedLocationPtr = std::unique_ptr<FramedLocation>;
    using LocationPtr = std::unique_ptr<Location>;

} // namespace armarx::priorknowledge::util
