#include "aron_conversions.h"

#include <RobotAPI/libraries/ArmarXObjects/aron_conversions.h>
#include <RobotAPI/libraries/armem/core/aron_conversions.h>
#include <RobotAPI/libraries/aron/common/aron_conversions.h>

namespace armarx::armem
{

    void
    fromAron(const arondto::ObjectInstance& dto, objpose::arondto::ObjectPose& bo)
    {
        bo = dto.pose;
    }

    void
    toAron(arondto::ObjectInstance& dto, const objpose::arondto::ObjectPose& bo)
    {
        dto.pose = bo;
    }

    void
    fromAron(const arondto::ObjectInstance& dto, objpose::ObjectPose& bo)
    {
        objpose::fromAron(dto.pose, bo);
    }

    void
    toAron(arondto::ObjectInstance& dto, const objpose::ObjectPose& bo)
    {
        objpose::toAron(dto.pose, bo);
    }

    /* Attachments */
    void
    fromAron(const arondto::attachment::AgentDescription& dto, attachment::AgentDescription& bo)
    {
        fromAron(dto.id, bo.id);
        aron::fromAron(dto.frame, bo.frame);
    }

    void
    toAron(arondto::attachment::AgentDescription& dto, const attachment::AgentDescription& bo)
    {
        toAron(dto.id, bo.id);
        aron::toAron(dto.frame, bo.frame);
    }

    void
    fromAron(const arondto::attachment::ObjectAttachment& dto, attachment::ObjectAttachment& bo)
    {
        fromAron(dto.agent, bo.agent);
        aron::fromAron(dto.transformation, bo.transformation);
        fromAron(dto.object, bo.object);
        aron::fromAron(dto.active, bo.active);
        // TODO aron::fromAron(dto.timestamp, bo.timestamp);
    }

    void
    toAron(arondto::attachment::ObjectAttachment& dto, const attachment::ObjectAttachment& bo)
    {
        toAron(dto.agent, bo.agent);
        aron::toAron(dto.transformation, bo.transformation);
        toAron(dto.object, bo.object);
        aron::toAron(dto.active, bo.active);
        // TODO aron::toAron(dto.timestamp, bo.timestamp);
    }

    void
    fromAron(const arondto::attachment::ArticulatedObjectAttachment& dto,
             attachment::ArticulatedObjectAttachment& bo)
    {
        fromAron(dto.agent, bo.agent);
        aron::fromAron(dto.transformation, bo.transformation);
        fromAron(dto.object, bo.object);
        aron::fromAron(dto.active, bo.active);
        // TODO aron::fromAron(dto.timestamp, bo.timestamp);
    }

    void
    toAron(arondto::attachment::ArticulatedObjectAttachment& dto,
           const attachment::ArticulatedObjectAttachment& bo)
    {
        toAron(dto.agent, bo.agent);
        aron::toAron(dto.transformation, bo.transformation);
        toAron(dto.object, bo.object);
        aron::toAron(dto.active, bo.active);
        // TODO aron::toAron(dto.timestamp, bo.timestamp);
    }

    void
    toAron(arondto::Marker& dto, const marker::Marker& bo)
    {
        dto.name = bo.name;
        armarx::toAron(dto.robotGlobal, bo.robotGlobal);
        armarx::toAron(dto.rgbCamera, bo.rgbCamera);
        armarx::toAron(dto.depthCamera, bo.depthCamera);
        armarx::toAron(dto.markerPose, bo.markerPose);
        armarx::toAron(dto.markerGlobal, bo.getGlobalMarkerPose());
    }

    void
    fromAron(const arondto::Marker& dto, marker::Marker& bo)
    {
        bo.name = dto.name;
        armarx::fromAron(dto.robotGlobal, bo.robotGlobal);
        armarx::fromAron(dto.rgbCamera, bo.rgbCamera);
        armarx::fromAron(dto.depthCamera, bo.depthCamera);
        armarx::fromAron(dto.markerPose, bo.markerPose);
    }

} // namespace armarx::armem

namespace armarx::armem::clazz
{

    void
    fromAron(const arondto::Feature& dto, Feature& bo)
    {
        bo.angle = dto.angle;
        bo.scale = dto.scale;
        bo.feature = dto.feature;
        bo.point2d = dto.point2d;
        bo.point3d = dto.point3d;
    }

    void
    toAron(arondto::Feature& dto, const Feature& bo)
    {
        dto.angle = bo.angle;
        dto.scale = bo.scale;
        dto.feature = bo.feature;
        dto.point2d = bo.point2d;
        dto.point3d = bo.point3d;
    }

    void
    fromAron(const arondto::ObjectClass& dto, ObjectClass& bo)
    {
        armarx::fromAron(dto.id, bo.id);
        armarx::fromAron(dto.simoxXmlPath, bo.simoxXmlPath);
        armarx::fromAron(dto.articulatedSimoxXmlPath, bo.articulatedSimoxXmlPath);
        armarx::fromAron(dto.urdfPath, bo.urdfPath);
        armarx::fromAron(dto.articulatedUrdfPath, bo.articulatedUrdfPath);
        armarx::fromAron(dto.sdfPath, bo.sdfPath);
        armarx::fromAron(dto.meshWrlPath, bo.meshWrlPath);
        armarx::fromAron(dto.meshObjPath, bo.meshObjPath);
        armarx::fromAron(dto.aabb, bo.aabb);
        armarx::fromAron(dto.oobb, bo.oobb);
        bo.ivtFeatures.clear();
        for (const auto& i : dto.ivtFeatures)
        {
            fromAron(i, bo.ivtFeatures.emplace_back());
        }
    }

    void
    toAron(arondto::ObjectClass& dto, const ObjectClass& bo)
    {
        armarx::toAron(dto.id, bo.id);
        armarx::toAron(dto.simoxXmlPath, bo.simoxXmlPath);
        armarx::toAron(dto.articulatedSimoxXmlPath, bo.articulatedSimoxXmlPath);
        armarx::toAron(dto.urdfPath, bo.urdfPath);
        armarx::toAron(dto.articulatedUrdfPath, bo.articulatedUrdfPath);
        armarx::toAron(dto.sdfPath, bo.sdfPath);
        armarx::toAron(dto.meshWrlPath, bo.meshWrlPath);
        armarx::toAron(dto.meshObjPath, bo.meshObjPath);
        armarx::toAron(dto.aabb, bo.aabb);
        armarx::toAron(dto.oobb, bo.oobb);
        dto.ivtFeatures.clear();
        for (const auto& i : bo.ivtFeatures)
        {
            toAron(dto.ivtFeatures.emplace_back(), i);
        }
    }

} // namespace armarx::armem::clazz

armarx::armem::MemoryID
armarx::armem::obj::makeObjectInstanceMemoryID(const objpose::ObjectPose& objectPose)
{
    return MemoryID("Object/Instance")
        .withProviderSegmentName(objectPose.providerName)
        .withEntityName(objectPose.objectID.str())
        .withTimestamp(objectPose.timestamp);
}
