#pragma once

#include <tuple>

#include <RobotAPI/libraries/skills/core/Skill.h>

namespace armarx
{
    namespace skills
    {
        class SkillBlueprint
        {
        public:
            using FunctionTypeToCreateSkill = std::function<std::unique_ptr<Skill>()>;

            SkillBlueprint(const FunctionTypeToCreateSkill& s) : _createSkill(s)
            {
            }

            SkillBlueprint(FunctionTypeToCreateSkill&& s) : _createSkill(std::move(s))
            {
            }

            SkillBlueprint(const SkillBlueprint&) = default;

            virtual ~SkillBlueprint() = default;

            template <class _Skill, class... Args>
            requires isSkill<_Skill>

            static std::unique_ptr<SkillBlueprint>
            ForSkill(Args&&... args)
            {
                auto _createSkill = [=]()
                {
                    auto ptr = std::make_unique<_Skill>(args...);
                    return ptr;
                };

                auto ret = std::make_unique<SkillBlueprint>(std::move(_createSkill));
                return ret;
            }

            virtual std::unique_ptr<Skill>
            createSkill(const skills::ProviderID& pid) const
            {
                auto s = _createSkill();
                s->setProviderId(pid);
                return s;
            }

            virtual SkillDescription
            createSkillDescription(const skills::ProviderID& pid) const
            {
                auto s = createSkill(pid);
                return s->getSkillDescription();
            }


        protected:
            FunctionTypeToCreateSkill _createSkill;
        };

        template <class T>
        concept isSkillBlueprint = std::is_base_of<SkillBlueprint, T>::value;

    } // namespace skills
} // namespace armarx
